#!/usr/bin/python3
"""
Created: June 2020
Author: amanno
Description: The conftest file for the EMS pipeline.  Creates test parameters and supplies fixtures.
"""
import constants
import helpers

import os
import pytest

POSTGRES_ENV = os.environ.get('POSTGRES_ENV', 'local')


def pytest_configure():
    """
    Configure the pytest runners.  Determine environment and gather correct parameters for that environment.
    Create parameter sets for each test to run and attach them to pytest object.
    :return: None
    """
    print('Detected Environment: {0}'.format(POSTGRES_ENV))
    if POSTGRES_ENV == 'dev01':
        pytest.valid_auth_params = constants.dev01_valid_auth_params
        pytest.valid_device_group_ids = constants.dev01_valid_device_group_ids
        pytest.deregistered_device_params = constants.dev01_deregistered_device_params
    elif POSTGRES_ENV == 'test03':
        pytest.valid_auth_params = constants.test03_valid_auth_params
        pytest.valid_device_group_ids = constants.test03_valid_device_group_ids
        pytest.deregistered_device_params = constants.test03_deregistered_device_params
    # We'll probably eventually want to make this conditional anything but local so it can run in any env
    elif POSTGRES_ENV == 'cb-emd1':
        pytest.valid_auth_params = constants.emd1_valid_auth_params
        pytest.valid_device_group_ids = constants.emd1_valid_device_group_ids
        pytest.deregistered_device_params = constants.emd1_deregistered_device_params
    else:
        pytest.valid_auth_params = constants.local_valid_auth_params
        pytest.valid_device_group_ids = constants.local_valid_device_group_ids
        pytest.deregistered_device_params = constants.local_deregistered_device_params

    pytest.valid_auth_env_params = helpers.make_valid_env_params(pytest.valid_auth_params)

    # 204 - Parameters
    pytest.valid_cache_device_id_params = helpers.make_valid_cache_params(pytest.valid_auth_params)
    pytest.valid_cache_org_id_params = helpers.make_valid_cache_params(pytest.valid_auth_params, False)
    pytest.cache_id_not_found_params = helpers.make_cache_id_not_found_params(pytest.valid_auth_params)

    # 404 - Parameters
    pytest.device_id_not_found_params = helpers.make_device_id_not_found_params(pytest.valid_auth_params)
    pytest.device_id_not_found_params_env = helpers.make_device_id_not_found_params(pytest.valid_auth_params, False)

    # 400 - Parameters
    pytest.bad_header_params = helpers.make_bad_header_params(pytest.valid_auth_params)
    pytest.bad_header_params_env = helpers.make_bad_header_params(pytest.valid_auth_params, False)

    pytest.bad_org_key_params = helpers.make_bad_org_key_params(pytest.valid_auth_params)
    pytest.bad_cache_id_params = helpers.make_bad_cache_id_params(pytest.valid_auth_params)

    # 500 - Parameters
    pytest.params_wrong_env = helpers.make_wrong_env_params(pytest.valid_auth_params)


@pytest.fixture
def supply_base_url() -> str:
    """
    Fixture used by all test cases to access the base URL per environment.
    :return: The base URL for the current environment
    """
    branch_name = os.environ.get('BRANCH_NAME', None)
    if branch_name is None or branch_name == 'LOCAL':
        return 'http://psc.ems/ems'
    if branch_name.lower() == 'master' or branch_name == "":
        return 'http://mcs-psc.cb-psc.io/ems'
    if branch_name.lower() == 'cb-emd1':
        return 'http://ems-device-auth:5004/ems'
    return f'http://mcs-psc.cb-psc.io/ems-{branch_name}'


@pytest.fixture
def supply_main_url(supply_base_url: str) -> str:
    """
    Fixture for supplying the URL for device-auth
    :param supply_base_url: Base URL for EMS
    :return: Device-Auth URL
    """
    return f'{supply_base_url}/v1/orgs'


@pytest.fixture
def supply_env_auth_url(supply_base_url: str) -> str:
    """
    Fixture for supplying the org-less URL for device-auth
    :param supply_base_url: Base URL for EMS
    :return: Org-less Device-Auth URL
    """
    return f'{supply_base_url}/v1/device_auth'
