#!/usr/bin/python3
"""
Created: January 2020
Author: amanno
Description: A helper script to encode and decode ORG KEYS to/from the Environment and Org ID.
"""
from hashids import Hashids

SALT = "SECRET_CB_SALT_TO_MAKE_DECODING_THESE_IDS_MORE_DIFFICULT"
ALPHABET = "ABCDEFGHJKLMNPQRSTUVWXYZ23456789"
MIN_LENGTH = 8
ENV_MAP = {
	"prod01": 1, "prod02": 2, "prod03": 3, "prod04": 4, "prod05": 5, "prod06": 6, "prodnrt": 7, "eap01": 21,
	"test02": 32, "test03": 33, "dev01": 41, "dev04": 44, "dev05": 45, "dev08": 48, "dev09": 49, "dev10": 50,
	"perf01": 51, "perf02": 52, "perf03": 53, "local": 60, "unittest": 61, "cb-emd1": 62
}


class Hasher:
	"""
	Object that can be used to encode and decode org_keys.
	"""

	def __init__(self, salt: str = SALT, alphabet: str = ALPHABET, min_length: int = MIN_LENGTH, env_map: dict = ENV_MAP, verbose: bool = True):
		"""
		Setup the hashing object.
		:param salt: Salt for the hashing algorithum
		:param alphabet: Allowed characters to use
		:param min_length: Smallest amount of characters encoded org_keys should have
		:param env_map: Mapping environments to numerical values.
		:param verbose: Enabling logging
		"""
		self.salt = salt
		self.alphabet = alphabet
		self.min_length = min_length
		self.env_map = env_map
		self.verbose = verbose
		self.hashids = Hashids(salt=self.salt, alphabet=self.alphabet, min_length=self.min_length)

	def encode(self, env: str, org_id: int) -> str:
		"""
		Encode the environment value and org id.
		:param env: Current environment - should be in ENV_MAP
		:param org_id: Org ID to encode with env
		:return: Org Key
		"""
		env_id = self.env_map[env]
		org_key = self.hashids.encode(env_id, org_id)
		if self.verbose:
			print("ORG ID: {0} + ENV: {1} ==> ORG_KEY: {2}".format(org_id, env, org_key))
		return org_key

	def decode(self, org_key: str) -> tuple:
		"""
		Decode the org key into its environment and org Id.
		:param org_key: Org Key to decode
		:return: Environment for org, Id for org
		"""
		env_id, org_id = self.hashids.decode(org_key)
		env = list(self.env_map.keys())[list(self.env_map.values()).index(env_id)]
		if self.verbose:
			print('ORG_KEY: {0} ==> ORG_ID: {1} + ENV: {2}'.format(org_key, org_id, env))
		return env, org_id

	def get_env_from_key(self, org_key: str) -> str:
		"""
		Get ONLY the environment from the org key.
		:param org_key: Org key to get env from.
		:return: Environment of org_key
		"""
		env, org_id = self.decode(org_key)
		if self.verbose:
			print('ORG_KEY: {0} has Environment "{1}"'.format(org_key, env))
		return env

	def get_org_id_from_key(self, org_key: str) -> int:
		"""
		Get ONLY the org Id from the org key.
		:param org_key: Org key to get the Id from
		:return: Org Id
		"""
		env, org_id = self.decode(org_key)
		if self.verbose:
			print('ORG_KEY: {0} has an ORG_ID of "{1}"'.format(org_key, org_id))
		return org_id
