#!/usr/bin/python3
"""
Created: June 2020
Author: amanno
Description: Helper functions to create new and/or fuzzed parameterized list of values to test against.
"""
from constants import Strings, FuzzList
from hasher import ENV_MAP

import os
import random
from psc_orgkey import encode_orgkey, decode_orgkey


# Common helpers
def get_org_id(reg_id: str) -> int:
    """
    Split the registration-id and return the Org Id only.
    :param reg_id: Device Registration-ID
    :return: Org Id
    """
    return int(reg_id.split('-')[0])


def get_device_id(reg_id: str) -> int:
    """
    Split the registration-id and return the Device Id only.
    :param reg_id: Device Registration-ID
    :return: Device Id
    """
    return int(reg_id.split('-')[1])


def convert_env_to_org_key(reg_id: str, env: str) -> str:
    """
    Using the registration-id and the environment get the org key.
    :param reg_id: Device Registration-ID
    :param env: Device Environment
    :return: Org Key
    """
    org_id = get_org_id(reg_id)
    return encode_orgkey(env, org_id)


def convert_org_key_to_env(org_key: str) -> str:
    """
    Decode the org_key into environment.
    :param org_key: Device Org Key
    :return: Device Environment
    """
    env, org_id = decode_orgkey(org_key)
    return env


# 200 Responses
def make_valid_env_params(auth_params: list) -> list:
    """
    Convert the org_key into the correct environment, used for the org-less endpoint.
    Example: [(<reg-id), <guid>, <org-key>)] ==> [(<reg-id>, <guid>, <env>)]
    :param auth_params: valid_auth_params with org_key as the 3rd element
    :return: Valid auth params with ENV instead of org_key.
    """
    valid_env_params = list()
    for param in auth_params:
        _validate_param(param)

        print(f'1. --------- {param[2]}')

        _p = (param[0], param[1], convert_org_key_to_env(param[2]))

        print(f'2. ---------- {_p}')
        valid_env_params.append(_p)

    if len(valid_env_params) != len(auth_params):
        raise ValueError('Was unable to make some valid ENV params')
    return valid_env_params


# 204 Responses
def make_valid_cache_params(auth_params: list, use_device_id: bool = True) -> list:
    """
    Make valid parameters for testing cache eviction.  Parse the registration-ids into device-ids/org-ids.
    Example: [(<reg-id>, <guid>, <org-key>)] ==> [(<device-id/org-id>, <org-key>)]
    :param auth_params: valid_auth_params from env
    :param use_device_id: Use device-id from reg-id instead of org-id
    :return: valid parameters for testing cache evictions
    """
    valid_cache_params = list()
    for param in auth_params:
        _validate_param(param)

        if use_device_id:
            _id = get_device_id(param[0])
        else:
            _id = get_org_id(param[0])

        _p = (_id, param[2])
        valid_cache_params.append(_p)
    return valid_cache_params


def make_cache_id_not_found_params(auth_params: list) -> list:
    """
    Make cache parameters that result in the device-id not being found in the database.
    Example: [(<reg-id>, <guid>, <org-key>)] ==> [(0, <org-key>)]
    :param auth_params: valid auth parameters for current env
    :return: parameters for testing cache eviction of device-ids not found
    """
    cache_params = list()
    for param in auth_params:
        _validate_param(param)

        _p = (0, param[2])
        cache_params.append(_p)

    return cache_params


# 400 Responses
def make_bad_header_params(auth_params: list, use_orgkey: bool = True) -> list:
    """
    Fuzz the values that go in the request headers (registration-id and guid).
    Example: [(<reg-id>, <guid>, <org-key>)] ==>
    [(<fuzzed-reg-id>, <guid>, <org-key/env>), (<reg-id>, <fuzzed-guid>, <org-key/env>)]
    :param auth_params: valid auth parameters for current env
    :param use_orgkey: use the org-key instead of the environment
    :return: parameters that contain fuzzed values for registration-id and guid
    """
    bad_headers = list()
    param = auth_params[0]
    _validate_param(param)

    org = param[2]
    if not use_orgkey:
        org = convert_org_key_to_env(param[2])

    for char in FuzzList.BAD_REG_IDS:
        if char == 'A':
            _invalid_reg_id = '{0}-A'.format(get_org_id(param[0]))
        elif char == 'B':
            _invalid_reg_id = 'B-{0}'.format(get_device_id(param[0]))
        elif char == '=':
            _invalid_reg_id = param[0].replace('-', '=')
        else:
            _invalid_reg_id = char

        _p = (_invalid_reg_id, param[1], org)
        bad_headers.append(_p)

    for char in FuzzList.BAD_GUIDS:
        _p = (param[0], char, org)
        bad_headers.append(_p)

    return bad_headers


def make_bad_org_key_params(auth_params: list) -> list:
    """
    Fuzz the org-key values for valid parameters.
    Example: [(<reg-id>, <guid>, <fuzzed-org-key>)]
    :param auth_params: valid auth parameters for current env
    :return: Parameters with fuzzed values for the org-key
    """
    bad_org_keys_list = list()
    param = auth_params[0]
    _validate_param(param)

    for fuzz in FuzzList.BAD_ORG_KEYS:
        _p = (param[0], param[1], fuzz)
        bad_org_keys_list.append(_p)
    return bad_org_keys_list


def make_bad_cache_id_params(auth_params: list) -> list:
    """
    Fuzz the device-ids for testing cache eviction.
    Example: [(<fuzzed-device-id>, <org-key>)]
    :param auth_params: valid auth parameters for current env
    :return: Parameters with fuzzed values for device-id
    """
    bad_cache_ids = list()
    param = auth_params[0]
    _validate_param(param)

    for fuzz in FuzzList.BAD_REG_IDS:
        _p = (fuzz, param[2])
        bad_cache_ids.append(_p)
    return bad_cache_ids


# 404 Responses
def make_device_id_not_found_params(auth_params: list, use_orgkey: bool = True) -> list:
    """
    For testing device-auth create a list of parameters where the device-ids are not fuzzed but not found in the db.
    Example: [(<org-id-0>, <guid>, <org-key/env>)]
    :param auth_params: valid auth parameters for current env
    :param use_orgkey: Use the org-key instead of environment
    :return: Parameters where device-id will not be found in the database
    """
    device_id_not_found_params = list()
    for param in auth_params:
        _validate_param(param)

        org = param[2]
        if not use_orgkey:
            org = convert_org_key_to_env(param[2])

        _new_reg_id = '{0}-{1}'.format(get_org_id(param[0]), 0)

        _p = (_new_reg_id, param[1], org, Strings.APP_JSON)
        device_id_not_found_params.append(_p)

        _p = (_new_reg_id, param[1], org, Strings.APP_PROTO)
        device_id_not_found_params.append(_p)

    if len(device_id_not_found_params) != len(auth_params) * 2:
        raise ValueError('Some params did not get made for invalid device ids')
    return device_id_not_found_params


# 500 Responses
def make_wrong_env_params(auth_params: list) -> list:
    """
    Select the incorrect environment to verify a 500 is returned from EMS
    Example: [(<reg-id>, <guid>, <incorrect-env>)]
    :param auth_params: valid auth parameters for current env
    :return: Parameters with the environment incorrect
    """
    actual_env = os.environ.get('POSTGRES_ENV', 'local')
    if actual_env not in ENV_MAP:
        raise ValueError(f'{actual_env} is NOT a recognized environment')

    params_wrong_env = list()
    for param in auth_params:
        _validate_param(param)

        env = actual_env
        while env == actual_env:
            env = random.choice(list(ENV_MAP.keys()))

        _p = (param[0], param[1], env)
        params_wrong_env.append(_p)

    if len(params_wrong_env) != len(auth_params):
        raise ValueError('Was unable to make some invalid ENVs')
    return params_wrong_env


# Privates
def _validate_param(param: tuple) -> bool:
    """
    Validate that a param object is of type tuple and has at least 3 entries: registration-id, GUID, org-key/env.
    :param param: Test parameter to validate
    :return: Bool
    """
    if not isinstance(param, tuple):
        raise TypeError(f'Parameters should be store in tuples not {type(param)}')

    if len(param) < 3:
        raise ValueError('All Parameters should have at least 3 entries: 1) registration-id 2) GUID 3) Env/OrgKey.')
    return True
