#!/usr/bin/python3
"""
Created: June 2020
Author: amanno
Description: This test will cover valid device-auth request to EMS, it coveres both routes (org-key, and org-key-less)
  for EMS in additional to testing both JSON and Protobuf.
"""
from helpers import get_device_id, convert_env_to_org_key
from constants import Strings, Lists
import device_auth_pb2 as pb

import json
import base64
import pytest
import requests


def verify_headers(resp: requests.models.Response, content_type: str = None):
    """
    Verify the headers from any device-auth request are a status code of 200 and contain the "Cache-Hit" header.
    Additionally verify the 'Accept' parameter matches what was sent.
    :param resp: The response object received back from the request
    :param content_type: The type of content to accept (I.E. json or protobuf)
    :return: None
    """
    assert resp.ok, f'Expected a 200 status code but instead got {resp.status_code}'
    assert resp.headers.get(Strings.CACHE_HIT) is not None, f'Unable to find header "Cache-Hit"'
    if content_type:
        assert resp.headers.get(Strings.CONTENT_TYPE) == content_type, f'Expected Response Content-Type is Incorrect'


def verify_valid_json(auth_data: dict, reg_id: str, guid: str, org_key: str):
    """
    Verify the JSON object returned from a request has the correct information that was sent.
    :param auth_data: JSON data from the response
    :param reg_id: Registration-ID that was sent in the request
    :param guid: Device-GUID that was sent in the request
    :param org_key: Org-Key that was sent in the request
    :return: None
    """
    assert auth_data['organization']['orgKey'] == org_key, 'Response data has the wrong ORG_KEY'
    assert auth_data['device']['id'] == get_device_id(reg_id), 'Response data has the wrong DEVICE_ID'
    assert auth_data['device']['deviceGuid'] == guid, 'Response data has the wrong DEVICE-GUID'
    assert auth_data['device']['deviceType'] in Lists.DEVICE_TYPES_NAMES, 'JSON response device type is invalid'
    assert auth_data['device']['deploymentType'] in Lists.DEPLOYMENT_TYPES_NAMES, 'JSON response deployment type is invalid'


def verify_valid_protobuf(raw_context: bytes, reg_id: str, guid: str, org_key: str):
    """
    Load the raw bytes of the HTTP response into the EMS protobuf object
    :param raw_context: Raw bytes of the request.models.response object
    :param reg_id: Registration-ID sent in the request
    :param guid: Device-GUID sent in the request
    :param org_key: Org-Key sent in the request
    :return: None
    """
    proto = pb.Context()
    proto.ParseFromString(raw_context)

    assert proto.organization.org_key == org_key, 'Protobuf response has the wrong ORG_KEY'
    assert proto.device.id == get_device_id(reg_id), 'Protobuf response has the wrong DEVICE ID'
    assert proto.device.device_guid == guid, 'Protobuf response has the wrong DEVICE-GUID'
    if hasattr(proto.device, 'device_type'):
        assert proto.device.device_type in Lists.DEVICE_TYPES_NUMS, 'Proto response device type is invalid'

    # Validate deployment type for valid values (0, 1, 2)
    if hasattr(proto.device, 'deployment_type'):
        assert proto.device.deployment_type in Lists.DEPLOYMENT_TYPES_NUMS, 'Proto response deployment type is invalid'


def verify_context_field(proto_data: bytes, json_data: dict):
    """
    Validate that all fields returned in the protobuf context object are present in the returned JSON context object.
    NOTE: The protobuf object can be missing fields if the value for that field is missing/None/blank
    :param proto_data: Raw byte data to be parsed by the ems device auth protobuf object
    :param json_data: Dictionary object representing the return device auth context object
    :return None
    """
    proto = pb.Context()
    proto.ParseFromString(proto_data)

    for proto_field in proto.device.DESCRIPTOR.fields:
        assert proto_field.camelcase_name in json_data['device'], f'Unable to find {proto_field.camelcase_name} in JSON DEVICE context'

    for proto_field in proto.organization.DESCRIPTOR.fields:
        # Handle corner case
        if proto_field.camelcase_name == 'effectiveThEndpoints':
            assert 'effectiveTHEndpoints' in json_data['organization'], 'Unable to find effectiveTHEndpoints in JSON ORGANIZATION context'
        else:
            assert proto_field.camelcase_name in json_data['organization'], f'Unable to find {proto_field.camelcase_name} in JSON ORGANIZATION context'

    for proto_field in proto.device_group.DESCRIPTOR.fields:
        assert proto_field.camelcase_name in json_data['deviceGroup'], f'Unable to find {proto_field.camelcase_name} in JSON DEVICE GROUP context'


class TestAuth:

    hparams = "reg_id, guid, org_key"

    @pytest.mark.parametrize(hparams, pytest.valid_auth_params)
    def test_auth_json(self, supply_main_url: str, reg_id: str, guid: str, org_key: str):
        """
        Verify device-auth for valid devices that accepts JSON for the device context object
        :param supply_main_url: Base URL for device-auth
        :param reg_id: Registration-ID to send
        :param guid: Device-GUID to send
        :param org_key: Org-Key to send
        :return: None
        """
        headers = {Strings.REG_ID: reg_id, Strings.GUID: guid}
        url = f'{supply_main_url}/{org_key}/device_auth'
        auth_resp = requests.get(url, headers=headers)

        verify_headers(auth_resp, content_type=Strings.APP_JSON)
        verify_valid_json(auth_resp.json(), reg_id, guid, org_key)

    @pytest.mark.parametrize(hparams, pytest.valid_auth_params)
    def test_auth_protobuf(self, supply_main_url: str, reg_id: str, guid: str, org_key: str):
        """
        Verify device-auth for valid devices that accepts a Protobuf for the device context object
        :param supply_main_url: Base URL for device-auth
        :param reg_id: Registration-ID to send
        :param guid: Device-GUID to send
        :param org_key: Org-Key to send
        :return: None
        """
        headers = {Strings.REG_ID: reg_id, Strings.GUID: guid, Strings.ACCEPT: Strings.APP_PROTO}
        url = f'{supply_main_url}/{org_key}/device_auth'
        auth_resp = requests.get(url, headers=headers)

        verify_headers(auth_resp, content_type=Strings.APP_PROTO)
        verify_valid_protobuf(auth_resp.content, reg_id, guid, org_key)

    @pytest.mark.parametrize(hparams, pytest.valid_auth_params)
    def test_auth_context_object(self, supply_main_url: str, reg_id: str, guid: str, org_key: str):
        """
        Make two auth requests for the same device (one JSON and one PROTOBUF) and verify all fields returned in the
        protobuf context object are present in the JSON context object
        :param supply_main_url: Base URL for device-auth
        :param reg_id: Registration-ID to send
        :param guid: Device-GUID to send
        :param org_key: Org-Key of the device to auth
        :return: None
        """
        proto_headers = {Strings.REG_ID: reg_id, Strings.GUID: guid, Strings.ACCEPT: Strings.APP_PROTO}
        json_headers = {Strings.REG_ID: reg_id, Strings.GUID: guid, Strings.ACCEPT: Strings.APP_JSON}
        url = f'{supply_main_url}/{org_key}/device_auth'

        auth_resp_proto = requests.get(url, headers=proto_headers)
        assert auth_resp_proto.ok, f'Device Auth PROTO request failed: {auth_resp_proto.text}'

        auth_resp_json = requests.get(url, headers=json_headers)
        assert auth_resp_json.ok, f'Device Auth JSON request failed: {auth_resp_json.text}'

        json_data = auth_resp_json.json()
        assert json_data, f'Failed to parse auth response into JSON format'

        verify_context_field(auth_resp_proto.content, json_data)


class TestAuthEnv:

    hparams = 'reg_id, guid, env'

    @pytest.mark.parametrize(hparams, pytest.valid_auth_env_params)
    def test_auth_json(self, supply_env_auth_url: str, reg_id: str, guid: str, env: str):
        """
        Verify org-key-less device-auth for valid devices that accepts JSON for the device context object.
        :param supply_env_auth_url: Base ENV URL for device-auth
        :param reg_id: Registration-ID to send
        :param guid: Device-GUID to send
        :param env: Device-Environment to send
        :return: None
        """
        headers = {Strings.REG_ID: reg_id, Strings.GUID: guid, Strings.DEV_ENV: env}
        auth_resp = requests.get(supply_env_auth_url, headers=headers)

        verify_headers(auth_resp)
        assert auth_resp.headers.get(Strings.DEV_CONTEXT), f'Headers do NOT contain a {Strings.DEV_CONTEXT} object'

        auth_data = json.loads(auth_resp.headers[Strings.DEV_CONTEXT])
        verify_valid_json(auth_data, reg_id, guid, convert_env_to_org_key(reg_id, env))

    @pytest.mark.parametrize(hparams, pytest.valid_auth_env_params)
    def test_auth_protobuf(self, supply_env_auth_url: str, reg_id: str, guid: str, env: str):
        """
        Verify org-key-less device-auth for valid devices that accepts Protobuf for the device context object.
        :param supply_env_auth_url: Base ENV URL for device-auth
        :param reg_id: Registration-ID to send
        :param guid: Device-GUID to send
        :param env: Device-Environment to send
        :return: None
        """
        headers = {Strings.REG_ID: reg_id, Strings.GUID: guid, Strings.DEV_ENV: env, Strings.ACCEPT: Strings.APP_PROTO}
        auth_resp = requests.get(supply_env_auth_url, headers=headers)

        verify_headers(auth_resp)
        assert auth_resp.headers.get(Strings.DEV_CONTEXT) is not None, 'No Device Context found'

        # Base64 decode the device context
        decoded_context = base64.b64decode(auth_resp.headers.get(Strings.DEV_CONTEXT))
        verify_valid_protobuf(decoded_context, reg_id, guid, convert_env_to_org_key(reg_id, env))

    @pytest.mark.parametrize(hparams, pytest.valid_auth_env_params)
    def test_auth_context_object(self, supply_env_auth_url: str, reg_id: str, guid: str, env: str):
        """
        Make two auth requests for the same device (one JSON and one PROTOBUF) and verify all fields returned in the
        protobuf context object are present in the JSON context object
        :param supply_env_auth_url: Base ENV URL for device-auth
        :param reg_id: Registration-ID to send
        :param guid: Device-GUID to send
        :param env: Device-Environment to send
        :return: None
        """
        proto_headers = {Strings.REG_ID: reg_id, Strings.GUID: guid, Strings.DEV_ENV: env, Strings.ACCEPT: Strings.APP_PROTO}
        json_headers = {Strings.REG_ID: reg_id, Strings.GUID: guid, Strings.DEV_ENV: env, Strings.ACCEPT: Strings.APP_JSON}

        auth_resp_proto = requests.get(supply_env_auth_url, headers=proto_headers)
        assert auth_resp_proto.ok, f'Device auth env proto route failed: {auth_resp_proto.text}'

        auth_resp_json = requests.get(supply_env_auth_url, headers=json_headers)
        assert auth_resp_json.ok, f'Device auth env json route failed: {auth_resp_json.text}'
        assert auth_resp_json.headers.get(Strings.DEV_CONTEXT), f'Headers do NOT contain a {Strings.DEV_CONTEXT} object'

        json_data = json.loads(auth_resp_json.headers[Strings.DEV_CONTEXT])
        assert json_data, f'Failed to parse auth response into JSON format'

        decoded_context = base64.b64decode(auth_resp_proto.headers.get(Strings.DEV_CONTEXT))
        verify_context_field(decoded_context, json_data)
