#!/usr/bin/python3
"""
Created: June 2020
Author: amanno
Description: This test verifies proper status codes are returned from requests that are malformed or contain
  fuzzed values for some/all of the fields.
"""
from constants import Strings
from helpers import convert_org_key_to_env

import pytest
import requests


class TestAuth400:

    @pytest.mark.parametrize('org_key', ['R76KWHRN', 'XXXXXXXX', None, 1, 'X' * 32])
    def test_auth_no_headers(self, supply_main_url: str, org_key: str):
        """
        Verify a Bad Request is returned when no headers are sent with a request to device-auth
        :param supply_main_url: Base device-auth URL
        :param org_key: Org-Key to send
        :return: None
        """
        url = f'{supply_main_url}/{org_key}/device_auth'
        auth_resp = requests.get(url)
        assert auth_resp.status_code == 400, 'Request was malformed (no headers), status code should be 400'

    def test_auth_env_no_headers(self, supply_env_auth_url: str):
        """
        Verify a Bad Request is returned when no headers are sent with a request to the org-key-less device-auth route
        :param supply_env_auth_url: Base ENV device-auth URL
        :return: None
        """
        auth_resp = requests.get(supply_env_auth_url)
        assert auth_resp.status_code == 400, 'Request was malformed (no headers), expected a 400 status code'

    @pytest.mark.parametrize('reg_id, guid, org_key', pytest.bad_header_params)
    def test_auth_bad_headers(self, supply_main_url: str, reg_id: str, guid: str, org_key: str):
        """
        Verify a Bad Request is returned when Registration-ID or Device-GUID values are fuzzed.
        :param supply_main_url: Base device-auth URL
        :param reg_id: Fuzzed Registration-ID
        :param guid: Fuzzed Device-GUID
        :param org_key: Valid Org-Key
        :return: None
        """
        headers = {Strings.REG_ID: reg_id, Strings.GUID: guid}
        url = f'{supply_main_url}/{org_key}/device_auth'
        auth_resp = requests.get(url, headers=headers)
        assert auth_resp.status_code == 400, 'Request was malformed (Registration-ID/GUID), expected a 400 status code.'

    @pytest.mark.parametrize('reg_id, guid, env', pytest.bad_header_params_env)
    def test_auth_env_bad_headers(self, supply_env_auth_url: str, reg_id: str, guid: str, env: str):
        """
        Verify a Bad Request is returned if Registration-ID or Device-GUID values are fuzzed for the org-key-less route.
        :param supply_env_auth_url: Base ENV device-auth URL
        :param reg_id: Fuzzed Registration-ID
        :param guid: Fuzzed Device-GUID
        :param env: Valid Device-Environment
        :return: None
        """
        headers = {Strings.REG_ID: reg_id, Strings.GUID: guid, Strings.DEV_ENV: env}
        auth_resp = requests.get(supply_env_auth_url, headers=headers)
        assert auth_resp.status_code == 400, 'Request was malformed (Registration-ID/GUID), expected a 400 status code.'

    @pytest.mark.parametrize('reg_id, guid, org_key', pytest.bad_org_key_params)
    def test_auth_bad_org_key(self, supply_main_url: str, reg_id: str, guid: str, org_key: str):
        """
        Verify a Bad Request is returned when the Org-Key value is fuzzed.
        :param supply_main_url: Base device-auth URL
        :param reg_id: Registration-ID to send
        :param guid: Device-GUID to send
        :param org_key: Fuzzed Org-Key to send
        :return: None
        """
        headers = {Strings.REG_ID: reg_id, Strings.GUID: guid}
        url = f'{supply_main_url}/{org_key}/device_auth'
        auth_resp = requests.get(url, headers=headers)
        assert auth_resp.status_code == 400, 'Request was malformed (bad org key), expected a 400 status code.'

    @pytest.mark.parametrize('reg_id, guid, env', pytest.bad_org_key_params)
    def test_auth_env_bad_env(self, supply_env_auth_url, reg_id, guid, env):
        """
        Verify a Bad Request is returned when the Device-Environment header value is fuzzed.
        :param supply_env_auth_url: Base ENV device-auth URL
        :param reg_id: Registration-ID to send
        :param guid: Device-GUID to send
        :param env: Fuzzed Device-Environment to send
        :return: None
        """
        headers = {Strings.REG_ID: reg_id, Strings.GUID: guid, Strings.DEV_ENV: env}
        auth_resp = requests.get(supply_env_auth_url, headers=headers)
        assert auth_resp.status_code == 400, 'Request was malformed (bad env), expected a 400 status code.'


class TestAuth404:

    @pytest.mark.parametrize('reg_id, guid, org_key, accept', pytest.device_id_not_found_params)
    def test_auth_device_id_not_found(self, supply_main_url: str, reg_id: str, guid: str, org_key: str, accept: str):
        """
        Verify a Not Found is returned when the Registration-ID has a Device-ID not in the database.
        :param supply_main_url: Base device-auth URL
        :param reg_id: Registration-ID to send - with a Device-ID that is NOT in the database
        :param guid: Device-GUID to send
        :param org_key: Org-Key to send
        :param accept: Content-Type to accept - json or protobuf
        :return: None
        """
        url = f'{supply_main_url}/{org_key}/device_auth'
        headers = {Strings.REG_ID: reg_id, Strings.GUID: guid, Strings.ACCEPT: accept}
        auth_resp = requests.get(url, headers=headers)
        assert auth_resp.status_code == 404, f'{org_key} should have NOT been found'

    @pytest.mark.parametrize('reg_id, guid, env, accept', pytest.device_id_not_found_params_env)
    def test_auth_env_device_id_not_found(self, supply_env_auth_url: str, reg_id: str, guid: str, env: str, accept: str):
        """
        Verify a Not Found is returned when the Registration-ID has a Device-ID not in the database for the
        org-key-less route.
        :param supply_env_auth_url: Base ENV device-auth URL
        :param reg_id: Registration-ID to send - with a Device-ID that is NOT in the database
        :param guid: Device-GUID
        :param env: Device-Environment to send
        :param accept: Content-Type to accept - json or protobuf
        :return: None
        """
        headers = {Strings.REG_ID: reg_id, Strings.GUID: guid, Strings.DEV_ENV: env, Strings.ACCEPT: accept}
        auth_resp = requests.get(supply_env_auth_url, headers=headers)
        assert auth_resp.status_code == 404, 'Should NOT have been found'

    # TODO: Test bad urls - healthcheck vs health_check


# This should only run in Dev01 or test03 not locally
class TestAuth401:

    @pytest.mark.parametrize('reg_id, guid, org_key, accept', pytest.deregistered_device_params)
    def test_auth_deregistered_device(self, supply_main_url: str, reg_id: str, guid: str, org_key: str, accept: str):
        """
        Verify that deregistered devices get a 401 status code - signaling the uninstall command
        :param supply_main_url: Base device-auth URL
        :param reg_id: Registration-ID of the deregistered device
        :param guid: Device-GUID of the deregistered device
        :param org_key: Org-Key of the deregistered device
        :param accept: HTTP Accept Header
        :return: None
        """
        url = f'{supply_main_url}/{org_key}/device_auth'
        headers = {Strings.REG_ID: reg_id, Strings.GUID: guid, Strings.ACCEPT: accept}
        auth_resp = requests.get(url, headers=headers)
        assert auth_resp.status_code == 401, f'Device {reg_id} has been deregistered and should be returning a 401'

    @pytest.mark.parametrize('reg_id, guid, org_key, accept', pytest.deregistered_device_params)
    def test_auth_env_deregistered_device(self, supply_env_auth_url: str, reg_id: str, guid: str, org_key: str, accept: str):
        """
        Verify that deregistered devices get a 401 status code - signaling the uninstall command
        :param supply_env_auth_url: Base device-auth URL
        :param reg_id: Registration-ID of the deregistered device
        :param guid: Device-GUID of the deregistered device
        :param org_key: Org-Key of the deregistered device
        :param accept: HTTP Accept Header
        :return: None
        """
        env = convert_org_key_to_env(org_key)
        headers = {Strings.REG_ID: reg_id, Strings.GUID: guid, Strings.DEV_ENV: env, Strings.ACCEPT: accept}
        auth_resp = requests.get(supply_env_auth_url, headers=headers)
        assert auth_resp.status_code == 401, f'Device {reg_id} has been deregistered and should be returning a 401'
